﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml;

using PcmCore.Metier;

namespace PcmCore.Donnees
{
    /// <summary>
    /// Module chargé d'exporter les contacts dans différents formats
    /// </summary>
    public class ContactsExporter
    {
        private ContactsManager contactsManager;
        private string savePath;    // Emplacement de sauvegarde
        private List<Contact> listeContacts;
        private StreamWriter writer;

        public string SavePath
        {
            get { return savePath; }
            set { savePath = value; }
        }

        public ContactsExporter (ContactsManager contactsManager)
        {
            this.contactsManager = contactsManager;
            this.savePath = "";
            listeContacts = new List<Contact> ();
            listeContacts = contactsManager.AllContacts ();
        }

        public ContactsExporter (ContactsManager contactsManager, string savePath) 
            : this (contactsManager)
        {
            this.savePath = savePath;
        }

        /// <summary>
        /// Formatage basique d'un texte contenant l'ensemble des contacts d'un utilisateur
        /// </summary>
        /// <returns>Chaine de caractère représentant un document formaté</returns>
        public string TransformToText ()
        {
            string docStruct = "Liste des contacts de l'utilisateur " + contactsManager.Utilisateur.NomUtilisateur + "\n\n\n";

            for (int i = 0; i < listeContacts.Count; i++)
            {
                docStruct += "--- Contact n°" + i + "\n\n";
                docStruct += "Nom : " + listeContacts[i].Nom + "\n";
                docStruct += "Prénom : " + listeContacts[i].Prenom + "\n";
                docStruct += "Date de naissance : " + listeContacts[i].DateNaissance + "\n";
                docStruct += "Adresse : " + listeContacts[i].Adresse + "\n";
                docStruct += "Code Postal : " + listeContacts[i].CodePostal + "\n";
                docStruct += "Ville : " + listeContacts[i].Ville + "\n";
                docStruct += "Numéro de téléphone Fixe : " + listeContacts[i].NumTelFix + "\n";
                docStruct += "Numéro de téléphone Portable : " + listeContacts[i].NumTelPort + "\n";
                docStruct += "Adresse eMail : " + listeContacts[i].AdrMail + "\n\n";
            }

            docStruct += "--- Fin de la liste des contacts de l'utilisateur " + contactsManager.Utilisateur.NomUtilisateur;
            var s = docStruct.Length; // 801
            return docStruct;
        }

        /// <summary>
        /// Exporte la liste de contact au format texte
        /// Format de sortie .txt
        /// L'emplacement de sauvegarde est défini dans savePath
        /// </summary>
        public void ExportToTextFormat ()
        {
            if (savePath != string.Empty)
            {
                try
                {
                    writer = new StreamWriter (savePath);
                    writer.Write (TransformToText ());
                    writer.Flush ();
                }
                catch (Exception e) { Console.Error.WriteLine (e.Message); }
                finally 
                { 
                    if (writer != null)
                        writer.Close (); 
                }
            }
        }

        /// <summary>
        /// Exporte la liste de contact au format tXML
        /// Format de sortie .xml
        /// L'emplacement de sauvegarde est défini dans savePath
        /// </summary>
        public void ExportToXML ()
        {
            if (savePath != string.Empty)
            {
                XmlDocument xmlDocument = new XmlDocument ();

                string xmlStruct = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
                xmlStruct += "<contacts utilisateur=\"" + contactsManager.Utilisateur.NomUtilisateur + "\" id=\"" + contactsManager.Utilisateur.IdUtilisateur + "\">\n";
                foreach (Contact c in listeContacts)
                {
                    xmlStruct += "\t<contact>";
                    xmlStruct += "\t\t<nom>" + c.Nom + "</nom>\n";
                    xmlStruct += "\t\t<prenom>" + c.Prenom + "</prenom>\n";
                    xmlStruct += "\t\t<dateNaissance>" + c.DateNaissance + "</dateNaissance>\n";
                    xmlStruct += "\t\t<adresse>" + c.Adresse + "</adresse>\n";
                    xmlStruct += "\t\t<codePostal>" + c.CodePostal + "</codePostal>\n";
                    xmlStruct += "\t\t<ville>" + c.Ville + "</ville>\n";
                    xmlStruct += "\t\t<numeroFixe>" + c.NumTelFix + "</numeroFixe>\n";
                    xmlStruct += "\t\t<numeroPortable>" + c.NumTelPort + "</numeroPortable>\n";
                    xmlStruct += "\t\t<email>" + c.AdrMail + "</email>\n";
                    xmlStruct += "\t</contact>";
                }
                xmlStruct += "\n</contacts>";

                xmlDocument.LoadXml (xmlStruct);
                xmlDocument.Save (savePath);
            }
        }
    }
}
